%% Calculate parameterised poincare section
clear;
maple restart;

dim = 3;                      % Dimension of flow .. USER INPUT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
fix_3=(5.7-sqrt(5.7^2-.16))/.4;  %%% will be replaced by maple construct soon
fix_2=-fix_3;                    %%%% need to parameterize the flow
fix_1=.2*fix_3;                 %%% these are fixed points of rossler which is an example
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global fix;
fix = [fix_1 fix_2 fix_3]';    %  fixed point - user input as of now

syms y1 y2 y3;                %%% will be replaced by symbolic array/loop
f=[-y2-y3; y1+.2*y2; .2+y3*(y1-5.7)];
v=[y1,y2,y3];                                
A=jacobian(f,v);                        %%%symbolic jacobian
J=subs(subs(subs(A,y1,fix_1),y2,fix_2),y3,fix_3);       %%% jacobian evaluated at fixed point
[e,lambda]=eig(J);                                      %%% eigen values and vectors computed
clear y1 y2 y3 A J v f fix_1 fix_2 fix_3;
maple restart;

for i=1:1:dim,
    if((~isreal(lambda(i,i)))&(real(lambda(i,i)>0))),
        cmplx=1;                               %%% check for complex unstable eigen vectors
        break;
    end 
end

if (cmplx == 1),
    complexvector1=e(:,i);
    complexeigen1=lambda(i,i);
    for i=1:1:dim,                               %%% get complex conjugate vector 
        if (lambda(i,i)==conj(complexeigen1)),
            complexvector2=e(:,i);
            break;
        end
    end
    global psection_vector;
    psection_vector=(0.5)*(complexvector1+complexvector2); %% will define poincare section as plane containing vectors orthogonal to this
    clear complexvector1 complexvector2 cmplx complexeigen1;
    
    options = odeset('Events',@events_cmplx);             %%% define poincare section in "events"
    tret=[];
    yret=[];
    
    num=100;                                            %% number of initial points
    len=1e-3;                                          %% distance from fix where initial condition starts
    
    tstart=0;
    tfinal=100;                                       %%% stopping time --- try to cover 15-16 return times?
    intersection=20;                                  %% stopping condition is 'intersection' number of crossings of section by soln.
    
    init=fix+len*psection_vector;
    for j=1:1:2,
       [t,y,te,ye,ie] = ode45(@funtry,[tstart tfinal],init,options);
       yret=[yret,ye'];
       init=ye';                                 
    end   
    for i=1:1:num,
        yinit(:,i)=yret(:,1)+(yret(:,2)-yret(:,1))*(i-1)/num;  %% generate 'num' initial points between 1st and 2nd intersections of starting point 
    end
    yret=[];    
    for i=1:1:num,
        init=yinit(:,i);yret=[yret,yinit(:,i)];
        for j=1:1:intersection-1,
            [t,y,te,ye,ie] = ode45(@funtry,[tstart tfinal],init,options);
            yret=[yret,ye'];
            init=ye';                                 %% yret holds the return coordinates in a 2-D array...in blocks of size 'intersection'
        end                                           %% SO size of array = (num*intersection)Xdim
    end
    clear t y te ye tstart tfinal del tret param init ie j;
else
    %%%% what if all are real eigen vectors?? have a different events
    %%%% function...will write soon
end

for j=1:1:intersection,
    for i=1:1:num,
        ypoinc(i+(j-1)*num,:)=yret(:,j+(i-1)*intersection)';   %%% regroup poincare returns according to lines
    end
end
clear yret curvature orientation;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
curvature(1:num*intersection)=0;         
curvature=curvature';                           %% store curvature
orientation(1:num*intersection)=1;
orientation=orientation';                       %% store orientation, ie if computed curvature is true or not (orientation=0 if FALSE)

for i=num:1:num*intersection,
    if(i<num*intersection),

          ra=ypoinc(i-1,:);
          rb=ypoinc(i,:);                      %% compute centre and radius of osculating circle
          rc=ypoinc(i+1,:);
          M=[norm(ra-rb)^2 (ra-rb)*(rb-rc)';(ra-rb)*(rb-rc)' norm(rb-rc)^2];
          alpha=inv(M)*[(-0.5)*(norm(ra-rb))^2;(0.5)*(norm(rb)^2-norm(rc)^2)+ra*(rc-rb)'];
          r0=ra+alpha(1)*(ra-rb)+alpha(2)*(rb-rc);
          curvature(i)=norm(r0-ra);
          if((((rb-ra)*(rc-ra)'<0)|((rc-r0)*(ra-r0)'>(rb-r0)*(ra-r0)'))&(mod(i,num)~=0)&(mod(i-1,num)~=0)),
              orientation(i)=0;               %% flag computed curvature FALSE
          end
    end
end

% for i=1:1:5,
%     plot3(ypoinc([1+num*(i-1):num*i],1),ypoinc([1+num*(i-1):num*i],2),ypoinc([1+num*(i-1):num*i],3),'r');
clear alpha M ra rb rc lambda r0 yinit;

count=1;i=1;turnback=0;
while((count<3)&(i<num*intersection)),                 %% find TURNBACK based on orientation
    if(orientation(i)==0),
        turnback(count)=i;
        count=count+1;
    end
    i=i+1;
end
 if(count<3),
     disp('Need more points?');           %% need to quit the program?? assume all turnbacks based on curvature - not good :((
     offset=300;i=offset;
     count=1;                              %% resetting count
     while((count<3)&(i<num*intersection)),
         if((curvature(i)<curvature(i+1))&(curvature(i)<curvature(i-1))),
             turnback(count)=i;
             count=count+1;               %%find TURNBACK based on curvature
         end
         i=i+1;
     end       
 end 
 if(count<3),
     disp('Really Really Need more points...'); 
 end
clear count offset;

arc_length(1:turnback(1))=len;         
arc_length=arc_length'; 
for i=2:1:turnback(1),                                %% compute arc_length for primary part of unstable manifold
    arc_length(i)=arc_length(i-1)+norm(ypoinc(i-1,:)-ypoinc(i,:));
end

for i=turnback(1)+1:1:turnback(2),
    mindist=1e4;minindex=0;
    for j=1:1:turnback(1),
        dist=norm(ypoinc(i,:)-ypoinc(j,:));
        if(dist<mindist),
            mindist=dist;
            minindex=j;                             %%compute nearest neighbor for secondary part of manifold
        end
    end
    arc_length(i)=arc_length(minindex);
end

for i=1:1:turnback(2)-num,
    s_1(i)=arc_length(i);s_2(i)=arc_length(i+num);   %%  get return map
%     plot(arc_length(i),arc_length(i+num),'r-');
end
% figure;hold on;
% plot(s_1,s_2,'r-');
% xlabel('s_n');ylabel('s_{n+1}');
clear ypoinc orientation curvature arc_length mindist minindex psection_vector dist e turnback len intersection num;
figure;
% set(gcf,'Units','inches','Position',[0 0 3.25 3.25*8/10]);
% set(gcf,'PaperPosition',[0 0 3.25 3.25*8/10]);
% set(gcf,'PaperSize',[3.25,3.25]);
plot(s_1,s_2,'r-');
set(gca,'FontName','Times','FontSize',12);
xlabel('s_n');ylabel('s_{n+1}');
print -dpng -r0 '..\figs\retmap_apr05.png'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear temp;
% temp=spline(s_1,s_2);
temp=csaps(s_1,s_2);                            %% SPLINE interpolation for return map
% tempx=0:.1:10;
% plot(tempx,ppval(temp,tempx));
[s_max,i]=max(s_2);
scrit=s_1(i);
s_curr=scrit;
cycles=10;                            %% order of prime cycles to be found
itin='0';

for i=1:1:cycles+3,
    x=ppval(temp,s_curr);
    if(x>scrit),
        itin(i)='1';
    else                            %% get itinerary of critical point
        itin(i)='0';
    end
    s_curr = x;
end
% kappa=0;
% for i=1:1:cycles+3,
%     itin=ppval(temp,s_curr);
%     if(itin>scrit),                     %% get itinerary of critical point
%         kappa=kappa+2^(12-i);
%     end
%     s_curr=itin;
% end
clear x s_curr s_max scrit;

kappa=spatial(itin);                    %% corresponding spatial location
kappa_dec=(bin2dec(kappa))/(2^(cycles+3));

% clear cycle_fulltable results;
% cycle_fulltable(5,9)=0;                   % table holding all possible cycles excluding 1-cycle(fixed point)
% results(5,9)=0;                           % corresponding entry '0' if cycle does not exist
% 
% cycle_fulltable(1,1)=[1];                           % 2 cycle
% cycle_fulltable(2,1:2)=[1 3];                       % 3 cycle
% cycle_fulltable(3,1:3)=[1 3 7];                     % 4 cycle
% cycle_fulltable(4,1:6)=[1 3 5 7 11 15];             %5 cycle
% cycle_fulltable(5,1:9)=[1 3 5 7 11 13 15 23 31];    % 6 cycle
cycle=cycle_all(10);
cycle=prunecyc(cycle,kappa_dec,cycles);
fid=fopen('cycles.txt','w');
for i=1:1:cycles,
    for j=1:1:length(cycle{i}),
        fprintf(fid,'%s\n',dec2bin(cycle{i}(j),i));
    end
end
fclose(fid);
% cycle=cycle_all(5);
cycles=5;
kappa_dec=0.8945;
region=prune(kappa_dec,cycles);   
fid=fopen('cycles.txt','w');
for i=1:1:cycles,
    for j=1:1:length(region{i}),
        fprintf(fid,'%s\n',dec2bin(region{i}(j),i));
    end
end
fclose(fid);